
#include <iostream>
#include <cmath>
#include <cstdlib>
#include <ctime>
using namespace std;

#define ULONG unsigned long int
ULONG gcd(ULONG x, ULONG y)
{
    if (!x)
        return y;
    return gcd(y % x, x);
}
ULONG gcdExtended(ULONG a, ULONG b, ULONG *x, ULONG *y)
{
    if (a == 0)
    {

        *x = 0, *y = 1;
        return b;
    }
    ULONG x1, y1;
    ULONG g = gcdExtended(b % a, a, &x1, &y1);
    *x = y1 - (b / a) * x1;
    *y = x1;
    return g;
}
ULONG modularCongruence(ULONG a, ULONG m)
{
    ULONG x, y;
    gcdExtended(a, m, &x, &y);
    return (x % m + m) % m;
}
ULONG computeRsa(ULONG a, ULONG b, ULONG c)
{
    ULONG y, z, i, j, k;
    y = a;
    z = 1;
    j = ceil(log(b + 1) / log(2)) * 8;
    for (i = 1, k = 1; k <= j; k++, i = i << 1)
    {
        if (b & i)
        {
            z = z * y % c;
        }
        y = y * y % c;
    }
    return z;
}
bool isPrime(ULONG n)
{
    for (ULONG i = 2; i <= sqrt(n); i++)
    {
        if (n % i == 0)
            return false;
    }
    return true;
}

ULONG generateRandomPrime()
{
    ULONG prime;
    do
    {
        prime = rand() % 73 + 2;
    } while (!isPrime(prime));
    return prime;
}

int main()
{
    srand(time(NULL));

    ULONG p, q, n, m, e, d;

    p = generateRandomPrime();
    q = generateRandomPrime();

    n = p * q;
    m = (p - 1) * (q - 1);

    do
    {
        e = rand() % m + 2;
    } while (gcd(e, m) != 1);

    d = modularCongruence(e, m);

    cout << "P = " << p << " and Q = " << q << "\n\n";
    cout << "n = " << n << "\n";
    cout << "m = " << m << "\n";
    cout << "e = " << e << "\n";
    cout << "d = " << d << "\n\n";
    cout << "public key: [" << e << ", " << n << "]\n";
    cout << "private key: [" << d << ", " << n << "]\n\n";

    char inputChar;
    while (true)
    {
        cout << "Enter a character: ";
        cin >> inputChar;
        if (inputChar == 'q')
            break;

        ULONG encryptedValue = computeRsa(static_cast<ULONG>(inputChar), e, n);

        ULONG decryptedValue = computeRsa(encryptedValue, d, n);

        cout << "Plain text = " << static_cast<int>(inputChar) << "\n";
        cout << "crypto = " << encryptedValue << "\n";
        cout << "decrypted output = " << decryptedValue << " " << static_cast<char>(decryptedValue) << "\n\n";
    }
    return 0;
}